unit IMPL;


interface
uses
  Windows, ActiveX, Classes, ComObj, Dewesoft_TLB, IniFiles, Dialogs, StrUtils,
  Math, SysUtils, Controls,  Forms;

type
  TExport = class(TComObject, ICustomExport, ICustomExport2, IExportFrame)
  private
    FApp: IApp;
    FFileName: string;

    FAbsoluteTime: Boolean;
    FTimeIncrease: Double;
    FDataCount: Int64;

  protected
    // ICustomExport1
    function Get_FileName(out Value: WideString): HResult; stdcall;
    function Set_FileName(const Value: WideString): HResult; stdcall;
    function StartExport(const App: IApp): HResult; stdcall;
    function EndExport: HResult; stdcall;
    function Get_ExportType(out Value: ExportTypes): HResult; stdcall;
    function Get_AbsoluteTime(out Value: WordBool): HResult; stdcall;
    function Set_AbsoluteTime(Value: WordBool): HResult; stdcall;

    function StartDataFolder(const FolderName: WideString; AbsTime: TDateTime): HResult; stdcall;
    function EndDataFolder: HResult; stdcall;

    function StartInfo(const Info: WideString): HResult; stdcall;
    function WriteInfoString(const Description: WideString; const Value: WideString): HResult; stdcall;
    function WriteInfoInteger(const Description: WideString; Param: Integer): HResult; stdcall;
    function WriteInfoSingle(const Description: WideString; Value: Single): HResult; stdcall;
    function WriteInfoDate(const Description: WideString; Value: TDateTime): HResult; stdcall;
    function EndInfo: HResult; stdcall;

    function StartTimeField(const FieldName: WideString; const FieldUnit: WideString): HResult; stdcall;
    function StartDataField(const FieldName: WideString; const FieldUnit: WideString; ExportRate: Integer): HResult; stdcall;
    function EndHeader: HResult; stdcall;

    function StartValue(TimeValue: Double): HResult; stdcall;
    function StartAbsValue(DateTime: TDateTime): HResult; stdcall;
    function WriteValue(Value: Single): HResult; stdcall;
    function WriteAsyncValue(TimeStamp: Double; Value: Single): HResult; stdcall;
    function StartChannel(const FieldName: WideString; const FieldUnit: WideString; Async: WordBool): HResult; stdcall;
    function Get_Extension(out Value: WideString): HResult; stdcall;
    function Get_TimeIncrease(out Value: Double): HResult; stdcall;
    function Set_TimeIncrease(Value: Double): HResult; stdcall;

    function EndChannel: HResult; stdcall;
    function EndValue: HResult; stdcall;

    function Get_SupportsAsync(out Value: WordBool): HResult; stdcall;
    function Get_SupportsSRDiv(out Value: WordBool): HResult; stdcall;
    function Get_DataCount(out Value: Largeuint): HResult; stdcall;
    function Set_DataCount(Value: Largeuint): HResult; stdcall;
    // ICustomExport2
    function SetAbsMin(AbsMin: Single): HResult; stdcall;
    function SetAbsMax(AbsMax: Single): HResult; stdcall;
    function SetTrigOffset(TrigTime: Double): HResult; stdcall;
    function SetChannelColor(Color: Integer): HResult; stdcall;
    function SetRangeMax(Value: Single): HResult; stdcall;
    function SetRangeMin(Value: Single): HResult; stdcall;
    function WriteDoubleValue(Value: Double): HResult; stdcall;
    function WriteAsyncDoubleValue(TimeStamp: Double; Value: Double): HResult; stdcall;
    function GetDWTypeLibVersion(out Value: Integer): HResult; stdcall;
    function Get_SupportsDouble(out Value: WordBool): HResult; stdcall;
    function SetDoubleFloat(DoubleFloat: WordBool): HResult; stdcall;
    function StartEvents: HResult; stdcall;
    function StopEvents: HResult; stdcall;
    function WriteEvent(EventType: Integer; const EventTypeString: WideString; Time: Double;
                        const Comment: WideString): HResult; stdcall;
    function SetApp(const App: IApp): HResult; stdcall;
    function SetChannel(const Ch: IChannel): HResult; stdcall;

    // IExport frame
    procedure ShowFrame(Handle: Integer; out FrameHeight: Integer); safecall;
    procedure HideFrame; safecall;
    procedure Apply; safecall;
    procedure SetExpApp(const App: IApp); safecall;
  public
    procedure Initialize; override;
    destructor Destroy; override;
  end;

  TExportObjectFactory = class(TComObjectFactory)
  public
    procedure UpdateRegistry(Register: Boolean); override;
  end;

implementation

uses
  ComServ, Registry;

const
  //New GUID can be generated by <Ctrl> + <Shift> + <G>
  Class_Export: TGUID = '{CD986D8A-DE5F-4C38-BFE4-B9B7B6616DCD}';

procedure TExportObjectFactory.UpdateRegistry(Register: Boolean);
var
Reg: TRegistry;
begin
  inherited UpdateRegistry(Register);
  if Register then
  begin
    Reg := TRegistry.Create(KEY_ALL_ACCESS);
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.OpenKey('Software\Dewesoft\Exports\EmptyExport', True);
    Reg.WriteString('GUID', GUIDToString(Class_Export)); // make sure to create new GUID!
    Reg.WriteString('Name', 'Empty export (*.?)');
    Reg.WriteString('Extension', '?');
    Reg.WriteString('Version', '1.0');
    Reg.WriteString('Vendor', 'Dewesoft');
    Reg.WriteInteger('Dewesoft version', 1);
    Reg.Free;
  end else
  begin
    Reg := TRegistry.Create(KEY_ALL_ACCESS);
    Reg.RootKey := HKEY_LOCAL_MACHINE;
    Reg.DeleteKey('Software\Dewesoft\Exports\EmptyExport');
    Reg.Free;
  end;
end;

{ TExport }
function TExport.EndChannel: HResult;
begin
  // end channel (etChannelBased)
  Result := 0;
end;

function TExport.EndDataFolder: HResult;
begin
  // end data folder (time between start-stop trigger)
  Result := 0;
end;


function TExport.EndExport: HResult; 
begin
  // export ended
  Result := 0;
end;

function TExport.EndHeader: HResult;
begin
  // End header section
  Result := 0;
end;

function TExport.EndInfo: HResult;
begin
  // End info section
  Result := 0;
end;

function TExport.EndValue: HResult;
begin
  // end value (etValueBased)
  Result := 0;
end;

function TExport.Get_AbsoluteTime(out Value: WordBool): HResult;
begin
  // Absolute or relative time export?
  Value := FAbsoluteTime;
  Result := 0;
end;

function TExport.Get_DataCount(out Value: Largeuint): HResult;
begin
  // Return number of samples (function not implemented)
  Value := FDataCount;
  Result := 0;
end;

function TExport.Get_ExportType(out Value: ExportTypes): HResult;
begin
  // value or channel based type?
  Value := etValueBased;
  Result := 0;
end;

function TExport.Get_Extension(out Value: WideString): HResult;
begin
  // extension is read from registry
  Value := '';
  Result := 0;
end;

function TExport.Get_FileName(out Value: WideString): HResult;
begin
  // Get export filename
  Value := FFileName;
  Result := 0;
end;

function TExport.Get_SupportsAsync(out Value: WordBool): HResult;
begin
  // does the export support asynchronous channels (only valid for etChannelBased type)
  // if set to false, async channels will be interpolated to synchronous rate
  Value := False;
  Result := 0;
end;

function TExport.Get_SupportsSRDiv(out Value: WordBool): HResult;
begin
  // Plugin supports SR divider?
  // if set to false, missing samples will be automatically inserted
  Value := False;
  Result := 0;
end;

function TExport.Get_TimeIncrease(out Value: Double): HResult;
begin
  // get time increase (function not implemented)
  Value := FTimeIncrease;
  Result := 0;
end;

function TExport.Set_AbsoluteTime(Value: WordBool): HResult;
begin
  // set abs/rel time
  FAbsoluteTime := Value;
  Result := 0;
end;

function TExport.Set_DataCount(Value: Largeuint): HResult;
begin
  // Number of samples that will be sent from Dewesoft
  FDataCount := Value;
  Result := 0;
end;

function TExport.Set_FileName(const Value: WideString): HResult;
begin
  // set export filename? when?
  FFileName := Value;
  Result := 0;
end;

function TExport.Set_TimeIncrease(Value: Double): HResult;
begin
  // Time increase for etValueBased export
  FTimeIncrease := Value;
  Result := 0;
end;

function TExport.StartAbsValue(DateTime: TDateTime): HResult;
begin
  // Timestamp (absolute time)
  Result := 0;
end;

function TExport.StartChannel(const FieldName, FieldUnit: WideString; Async: WordBool): HResult;
begin
  // if etChannelBased, called for each start of the channel export
  Result := 0;
end;

function TExport.StartDataField(const FieldName, FieldUnit: WideString; ExportRate: Integer): HResult;
begin
  // Called for each channel (name, unit, export rate)
  Result:= 0;
end;

function TExport.StartDataFolder(const FolderName: WideString; AbsTime: TDateTime): HResult;
begin
  // start data folder (time between start-stop trigger)
  Result := 0;
end;

function TExport.StartExport(const App: IApp): HResult;
begin
  // start exporting
  // IApp object is already received with the "SetApp" function
  Result := 0;
end;

function TExport.StartInfo(const Info: WideString): HResult;
begin
  // Info section started
  Result := 0;
end;

function TExport.StartTimeField(const FieldName, FieldUnit: WideString): HResult;
begin
  // Name of the time channel and its unit
  Result := 0;
end;

function TExport.StartValue(TimeValue: Double): HResult;
begin
  // Timestamp (relative time)
  Result := 0;    
end;

function TExport.WriteAsyncValue(TimeStamp: Double; Value: Single): HResult;
begin
  // Write asynchronous channel value
  // called for etChannelBased type for asynchronous channels only
  Result := 0;
end;

function TExport.WriteInfoDate(const Description: WideString; Value: TDateTime): HResult;
begin
  // Write info variable (date type)
  Result := 0;
end;

function TExport.WriteInfoInteger(const Description: WideString; Param: Integer): HResult;
begin
  // Write info variable (integer type)
  Result := 0;
end;

function TExport.WriteInfoSingle(const Description: WideString; Value: Single): HResult;
begin
  // Write info variable (float type)
  Result := 0;
end;

function TExport.WriteInfoString(const Description, Value: WideString): HResult;
begin
  // Write info variable (string type)
  Result := 0;
end;

function TExport.WriteValue(Value: Single): HResult;
begin
  // Write channel value
  // called if etValueBased type (all channels) or for etChannelBased synchronous channels
  Result := 0;
end;

destructor TExport.Destroy;
begin
  inherited;
end;


function TExport.Get_SupportsDouble(out Value: WordBool): HResult;
begin
  // Supports double floating point
  Value := True;
  Result := 0;
end;

function TExport.GetDWTypeLibVersion(out Value: Integer): HResult;
begin
  Value := DEWEsoftMinorVersion;
  Result := 0;
end;

function TExport.SetAbsMax(AbsMax: Single): HResult;
begin
  Result := 0;
end;

function TExport.SetAbsMin(AbsMin: Single): HResult;
begin
  Result := 0;
end;

function TExport.SetApp(const App: IApp): HResult;
begin
  // Get reference to IApp object
  FApp := App;
  Result := 0;
end;

function TExport.SetChannel(const Ch: IChannel): HResult;
begin
  // Reference to channel
  Result := 0;
end;

function TExport.SetChannelColor(Color: Integer): HResult;
begin
  Result := 0;
end;

function TExport.SetDoubleFloat(DoubleFloat: WordBool): HResult;
begin
  // Channel supports double floating point precision?
  Result := 0;
end;

function TExport.SetRangeMax(Value: Single): HResult;
begin
  Result := 0;
end;

function TExport.SetRangeMin(Value: Single): HResult;
begin
  Result := 0;
end;

function TExport.SetTrigOffset(TrigTime: Double): HResult;
begin
  Result := 0;
end;

function TExport.StartEvents: HResult;
begin
  // Start events section
  Result := 0;
end;

function TExport.StopEvents: HResult;
begin
  // Stop events section
  Result := 0;
end;

function TExport.WriteAsyncDoubleValue(TimeStamp, Value: Double): HResult;
begin
  // Write asynchronous channel value (double precision)
  // called for etChannelBased type for asynchronous channels only
  // only used if "Get_SupportsDouble" is set to True
  Result := 0;
end;

function TExport.WriteDoubleValue(Value: Double): HResult;
begin
  // Write channel value (double precision)
  // called if etValueBased type (all channels) or for etChannelBased synchronous channels
  // only used if "Get_SupportsDouble" is set to True
  Result := 0;
end;

function TExport.WriteEvent(EventType: Integer; const EventTypeString: WideString; Time: Double; const Comment: WideString): HResult;
begin
  // Write single event
  Result := 0;
end;

procedure TExport.Initialize;
begin
  inherited;
end;

procedure TExport.Apply;
begin
  // apply changes on the frame
end;

procedure TExport.HideFrame;
begin
  // Set frame visible to false and set parenwindow handle to 0
end;

procedure TExport.SetExpApp(const App: IApp);
begin
  // IApp reference for frame
  FApp := App;
end;

procedure TExport.ShowFrame(Handle: Integer; out FrameHeight: Integer);
begin
  // Show custom form (set Handle as parent window of user frame)
end;

initialization
  TExportObjectFactory.Create(ComServer, TExport, Class_Export, 'Export', '', ciMultiInstance, tmApartment);
end.


