﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;
using Microsoft.Win32;

namespace VCSExportExample
{
    [Guid("4AE7FB5B-A62E-456B-95E0-4A19AC12CCEC")]
    public class VCSExportImpl : DEWEsoft.ICustomExport, DEWEsoft.ICustomExport2, DEWEsoft.ICustomExport3, DEWEsoft.IExportFrame
    {
        private string ExpFileName;
        System.IO.StreamWriter ExpFile;

        // constants
        private const string FRegKeyForPlugin = "VCSExportExample";
        private const string FPluginName = "VCS Export Example (*.txt)";
        private const string FPluginDesc = "Visual C# Export Example";
        private const int FDewesoftTypeLibVersion = 7;

        #region ComRegister

        [ComRegisterFunctionAttribute]
        public static void RegisterFunction(Type t)
        {
            String guidStr = "{" + t.GUID.ToString() + "}";
            RegistryKey regHKLM = Registry.LocalMachine;
            String keyName = "Software\\Dewesoft\\Exports\\" + FRegKeyForPlugin;

            regHKLM = regHKLM.CreateSubKey(keyName);
            regHKLM.OpenSubKey(keyName);
            regHKLM.SetValue("GUID", guidStr);
            regHKLM.SetValue("Name", FPluginName);
            regHKLM.SetValue("Extension", "txt");
            regHKLM.SetValue("Description", FPluginDesc);
            regHKLM.SetValue("Version", "1.0");
            regHKLM.SetValue("Vendor", "Dewesoft");
            regHKLM.SetValue("TLB", 7);
            regHKLM.SetValue("NET", 1);
        }

        [ComUnregisterFunctionAttribute]
        public static void UnregisterFunction(Type t)
        {
            RegistryKey regHKLM = Registry.LocalMachine;
            regHKLM.DeleteSubKey("Software\\Dewesoft\\Exports\\" + FRegKeyForPlugin);
        }

        #endregion

        #region ICustomExport Members

        public bool AbsoluteTime
        {
            get
            {
                throw new NotImplementedException();
            }
            set
            {
            }
        }

        public ulong DataCount
        {
            get
            {
                throw new NotImplementedException();
            }
            set
            {
            }
        }

        public void EndChannel()
        {
        }

        public void EndDataFolder()
        {
        }

        public void EndExport()
        {
            ExpFile.Close();
            ExpFile.Dispose();
        }

        public void EndHeader()
        {
            ExpFile.WriteLine();
        }

        public void EndInfo()
        {
        }

        public void EndValue()
        {
            ExpFile.WriteLine();
        }

        public DEWEsoft.ExportTypes ExportType
        {
            get
            {
                return DEWEsoft.ExportTypes.etValueBased;
            }
        }

        public string Extension
        {
            get { throw new NotImplementedException(); }
        }

        public string FileName
        {
            get
            {
                throw new NotImplementedException();
            }
            set
            {
                ExpFileName = value;
            }
        }

        public void StartAbsValue(DateTime DateTime)
        {
        }

        public void StartChannel(string FieldName, string FieldUnit, bool Async)
        {
        }

        public void StartDataField(string FieldName, string FieldUnit, int ExportRate)
        {
            ExpFile.Write(FieldName + "\t");
        }

        public void StartDataFolder(string FolderName, DateTime AbsTime)
        {
            ExpFile.WriteLine();
            ExpFile.WriteLine("Trigger...");
            ExpFile.WriteLine();
        }

        public void StartExport(DEWEsoft.App App)
        {
            ExpFile = new System.IO.StreamWriter(ExpFileName);
        }

        public void StartInfo(string Info)
        {
            ExpFile.WriteLine("File header:");
        }

        public void StartTimeField(string FieldName, string FieldUnit)
        {
            ExpFile.Write(FieldName + "\t");
        }

        public void StartValue(double TimeValue)
        {
            ExpFile.Write(TimeValue.ToString() + "\t");
        }

        public bool SupportsAsync
        {
            get
            {
                return true;
            }
        }

        public bool SupportsSRDiv
        {
            get
            {
                return true;
            }
        }

        public double TimeIncrease
        {
            get
            {
                throw new NotImplementedException();
            }
            set
            {
            }
        }

        public void WriteAsyncValue(double TimeStamp, float Value)
        {
        }

        public void WriteInfoDate(string Description, DateTime Value)
        {
        }

        public void WriteInfoInteger(string Description, int Param)
        {
        }

        public void WriteInfoSingle(string Description, float Value)
        {
        }

        public void WriteInfoString(string Description, string Value)
        {
            ExpFile.WriteLine(Description + ": " + Value);
        }

        public void WriteValue(float Value)
        {
            if (!Double.IsNaN(Value))
                ExpFile.Write(Value.ToString());
            ExpFile.Write("\t");
        }

        #endregion

        #region ICustomExport2 Members

        public int GetDWTypeLibVersion()
        {
            return 0;
        }

        public void SetAbsMax(float AbsMax)
        {
        }

        public void SetAbsMin(float AbsMin)
        {
        }

        public void SetApp(DEWEsoft.App App)
        {
        }

        public void SetChannel(DEWEsoft.IChannel Ch)
        {
        }

        public void SetChannelColor(int Color)
        {
        }

        public void SetDoubleFloat(bool DoubleFloat)
        {
        }

        public void SetRangeMax(float Value)
        {
        }

        public void SetRangeMin(float Value)
        {
        }

        public void SetTrigOffset(double TrigTime)
        {
        }

        public void StartEvents()
        {
            ExpFile.WriteLine();
            ExpFile.WriteLine("Event list:");
        }

        public void StopEvents()
        {
            ExpFile.WriteLine("End events.");
        }

        public bool SupportsDouble
        {
            get
            {
                return true;
            }
        }

        public void WriteAsyncDoubleValue(double TimeStamp, double Value)
        {
        }

        public void WriteDoubleValue(double Value)
        {
        }

        public void WriteEvent(int EventType, string EventTypeString, double Time, string Comment)
        {
            ExpFile.WriteLine("Event type: " + EventType.ToString() + ", Time: " + Time.ToString());
        }

        #endregion

        #region ICustomExport3 Members

        public int OnEvent(DEWEsoft.CustomExpEventIDs EventIDs, object InParam, ref object OutParam)
        {
            return 0;
        }

        #endregion

        #region IExportFrame Members

        public void Apply()
        {
        }

        public void HideFrame()
        {
        }

        public void SetExpApp(DEWEsoft.App App)
        {
        }

        public void ShowFrame(int Handle, out int FrameHeight)
        {
            FrameHeight = 0;
        }

        #endregion

    }
}
